# Templates

## Liste

### Utilisation minimale
```php
// Exemple sur liste utilisateurs
{!! Template::list3([
        'table_id'     => null,
        'table_class'  => 'table-sortable',
        'route_prefix' => $route_prefix,
        'can_prefix'   => $can_prefix,
        'menu'         => [
            'display'       => true,
            'is_readable'   => true,
            'is_editable'   => true,
            'is_deletable'  => true,
            'is_duplicable' => false,
        ],
        'source' => $items,
        'columns' => array([
	            'label'    => '#',
	            'value'    => ':id',
	            'sortable' => 'id'
	        ],[
	            'label'    => 'Dossier',
	            'value'    => ':folder_html',
	            'sortable' => 'ged_folders.title'
	        ],[
	            'label'    => 'Nom',
	            'value'    => ':name',
	            'sortable' => 'name',
	        ],[
	            'label'    => 'Version',
	            'value'    => ['template' => [
                    'format' => 'v%s',
                    'args'   => ':version'
                ]],
	            'sortable' => 'version',
	            'class'    => 'text-center'
	        ],[
	            'label'    => 'Statut',
	            'value'    => ':validation_html',
	            'class'    => 'text-center'
	        ],[
	            'label'    => 'Visibilité',
	            'class'    => 'text-center',
	            'value'    => ['callback' => [
                    'function' => 'published_html', 
                    'args'     => ':status'
                ]],
	            'sortable' => 'status',
	        ],[
	            'label'    => 'Date',
	            'value'    => ':updated_at->diffForHumans()',
	            'sortable' => 'updated_at'
	        ]
	    )
    ]) !!}
```

### Utilisation avancée
```php
// Exemple sur liste utilisateurs
{!! Template::list3([
        'table_id'     => null,
        'table_class'  => 'table-sortable',
        'route_prefix' => $route_prefix,
        'can_prefix'   => $can_prefix,
        'menu'         => [
            'display'       => true, // Affichage du menu
            'is_readable'   => true,
            'is_editable'   => true,
            'is_deletable'  => true,
            'is_duplicable' => false,
            'tags' => [
                ['template' => ['format' => '[ module|ged|document|%s ]', 'args' => ':id']],
                ['template' => ['format' => '[ module|ged|document|%s ]', 'args' => ['callback' => [
                    'function' => 'Str::slug', 'args' => ':name']]
                ]],
            ]
            'more' => array([
	                'label'  => '<i class="fa fa-unlock mr-1"></i> RAZ validation',
	                'href'   => [
	                    'template' => [
                            'format' => url('backend/module/ged/file/status/%s/0?' . current_query_to_string()),
                            'args'   => ':files->sortBy()->last()->id'
                        ],
	                ],
	            ]
	        )
        ],
        'source' => $items,
        'rows'   => array(
            'class' => [
                'condition' => [
                    'value'      => ':id',
                    'operator'   => '==',
                    'comparator' => 1,
                    'true'       => ':name',
                    'false'      => 'text-danger'
                ]
            ],
        ),
        'columns' => array(
           [
                'label'    => '#',
                'value'    => ':id',
                'sortable' => 'id'
            ],[
                'label'    => 'Poster',
                'value'    => ['template' => [
                    'format' => '<img class="thumb-list" src="%s" alt="%s"', 
                    'args'   => [':poster', ':name']
                ]],
                'sortable' => 'poster'
            ],[
                'label'    => 'Nom',
                'value'    => ':name',
                'sortable' => 'name'
            ],[
                'label'    => 'Description',
                'sortable' => 'description',
                'value'    => [
                    'callback' => [
                        'function' => 'str_limit',
                        'args'     => [
                            'callback' => [
                                'function' => 'strip_tags',
                                'args'     => [
                                    'callback' => [
                                        'function' => 'html_entity_decode',
                                        'args'     => ':description'
                                    ]
                                ]
                            ],
                            20
                        ]
                    ]
                ],
            ],[
				'label'    => 'Position',
				'class'    => 'text-center',
                'value'     => ['callback' => [
                    'function' => 'pos_sort', 
                    'args'     => [':id', ':position']
                ]],
				'sortable' => 'position'
			], [
                'label'    => 'Categories',
                'value'    => ':categories_html',
            ],[
                'label'    => 'Visibilité',
                'class'    => 'text-center',
                'value'    => ['callback' => [
                    'function' => 'published_html', 
                    'args'     => ':status'
                ]],
                'sortable' => 'status',
            ],[
                'label'    => 'Date',
                'value'    => ':updated_at->diffForHumans()',
                'sortable' => 'updated_at'
            ]
	    )
    ]) !!}
```
