<?php

namespace Goldenscarab\Modulus\Service\Template\Compose;

use Illuminate\Support\Str;
use \Illuminate\Support\HtmlString;
use Illuminate\Support\Facades\Request;


class Template1Bt4
{
    private $route_prefix;

    private $route_param;

    private $table_id;

    private $table_class;

    private $can_prefix;

    private $has_menu;

    private $is_editable;

    private $is_deletable;

    private $is_readable;

    private $is_duplicable;

    private $_add_row;

    private $_datas;

    private $_tags;

    private $_menu_items;



    public function __construct($attrs = [])
    {
        $this->setAttrs($attrs);
    }

    /**
     * Retourne le rendu d'un tableau liste
     * @return string le rendu du tableau liste
     */
    public function renderList(): string
    {
        $template  = '<div class="table-responsive-lg" style="min-height: 250px;">' . PHP_EOL;
        $template .= '    <table id="%s" class="%s">' . PHP_EOL;
        $template .= '        <thead>' . PHP_EOL;
        $template .= '            <tr>' . PHP_EOL;
        $template .=                '%s';
        $template .= '            </tr>' . PHP_EOL;
        $template .= '        </thead>' . PHP_EOL;
        $template .= '        <tbody>' . PHP_EOL;
        $template .=             '%s';
        $template .= '        </tbody>' . PHP_EOL;
        $template .= '    </table>';
        $template .= '</div>';


        $render = vsprintf($template, [
            empty($this->table_id) ? 'table-' . Str::slug($this->route_prefix) : $this->table_id,
            trim('table table-sm table-hover table-striped ' . $this->table_class),
            $this->renderIndent($this->renderListHead(), 3),
            $this->renderIndent($this->renderListBody(), 2),
        ]);

        return $render;
    }

    /**
     * Retourne le rendu des entêtes de colonnes d'un tableau liste
     * @return string Le rendu des entêtes de colonnes
     */
    public function renderListHead(): string
    {
        $template = '<th scope="col"%s>%s%s</th>' . PHP_EOL;

        $render = '';

        $columns = $this->_datas['columns'];

        // Colonne titre
        if (is_iterable($columns)) {
            foreach ($columns as $column) {
                $sortable = data_get($column, 'sortable');
                $class = data_get($column, 'class') . ' text-nowrap';
                $render .= vsprintf($template, [
                    $this->renderClass($class),
                    data_get($column, 'label'),
                    is_null($sortable) ? '' : col_sort($sortable)
                ]);
            }
        }

        // Colonne menu
        $render .= vsprintf($template, ['','', '']);

        return $render;
    }

    /**
     * Retourne le rendu des lignes d'un tableau liste
     * @return string Le rendu des lignes du tableau
     */
    public function renderListBody(): string
    {
        $template_row  = '<tr scope="row"%s>' . PHP_EOL;
        $template_row .=     '%s';
        $template_row .= '</tr>' . PHP_EOL;
        $template_cel  = '<td%s%s>%s</td>' . PHP_EOL;

        $render     = '';
        $collection = $this->_datas['collection'];
        $columns    = $this->_datas['columns'];

        if (is_iterable($collection)) {
            if (count($collection) == 0) {
                $render_cels_row = vsprintf($template_cel, [
                    ' colspan="' . (count($columns) + 1) .'"',
                    null,
                    'Aucun élément disponible'
                ]);

                $render .= vsprintf($template_row, [
                    null,
                    $this->renderIndent($render_cels_row, 1)
                ]);
            }

            foreach ($collection as $item) {
                $render_cels_row = '';
                if (is_iterable($columns)) {
                    foreach ($columns as $column) {

                        $template   = data_get($column, 'template');
                        $name       = data_get($column, 'name');
                        $callback   = data_get($column, 'callback');
                        $text       = $this->objectToString($template, $name, $callback, $item);

                        $attributes = data_get($column, 'attributes');
                        $class      = data_get($column, 'class');

                        // Création de la cellule
                        $render_cels_row .= vsprintf($template_cel, [
                            $this->renderAttributes($attributes, $item),
                            $this->renderClass($class),
                            $text
                        ]);
                    }

                    // Ajout du menu en fin de ligne
                    $render_cels_row .= vsprintf($template_cel, [
                        null,
                        null,
                        $this->has_menu ? $this->renderListMenuItems($item) : ''
                    ]);
                }

                $render .= vsprintf($template_row, [
                    $this->renderAddRow($item),
                    $this->renderIndent($render_cels_row, 1)
                ]);
            }
        }

        return $render;
    }

    /**
     * Retourne le rendu d'ajout à une ligne en fonction d'une condition
     * @param  Mix $item    L'élément concerné par la condition
     * @return string       Le rendu à rajouter à la ligne
     */
    private function renderAddRow($item): string
    {
        $render = '';

        if (count($this->_add_row) > 0) {

            $column   = data_get($this->_add_row, 'condition.0');
            $operator = data_get($this->_add_row, 'condition.1');
            $value    = data_get($this->_add_row, 'condition.2');

            switch ($operator) {
                case '==' :
                    $required_add = (data_get($item, $column) == $value ? true : false);
                    break;
                case '>' :
                    $required_add = (data_get($item, $column) > $value ? true : false);
                    break;
                case '<' :
                    $required_add = (data_get($item, $column) < $value ? true : false);
                    break;
                case '!=' :
                    $required_add = (data_get($item, $column) != $value ? true : false);
                    break;
                case '<=' :
                    $required_add = (data_get($item, $column) <= $value ? true : false);
                    break;
                case '>=' :
                    $required_add = (data_get($item, $column) >= $value ? true : false);
                    break;
                default :
                    $required_add = false;
            }

            if ($required_add) {
                $render = sprintf(' class="%s"', data_get($this->_add_row, 'class'));
            }
        }

        return $render;
    }

    private function renderListMenuItems($item): string
    {
        $template  = '<div class="dropdown">' . PHP_EOL;
        $template .= '    <button class="btn btn-default btn-xs dropdown-toggle no-caret square-to-circle item-option" type="button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">' . PHP_EOL;
        $template .= '        <i class="fa fa-cog" aria-hidden="true"></i>' . PHP_EOL;
        $template .= '    </button>' . PHP_EOL;
        $template .= '    <div class="dropdown-menu dropdown-menu-right">' . PHP_EOL;
        $template .= '%s';
        $template .= '%s';
        $template .= '%s';
        $template .= '%s';
        $template .= '%s';
        $template .= '%s';
        $template .= '    </div>' . PHP_EOL;

        $render = '';

        $render = vsprintf($template, [
            $this->renderIndent($this->renderListMenuItemRead($item), 2),
            $this->renderIndent($this->renderListMenuItemUpdate($item), 2),
            $this->renderIndent($this->renderListMenuItemTag($item), 2),
            $this->renderIndent($this->renderListMenuItemMore($item), 2),
            $this->renderIndent($this->renderListMenuItemDuplicate($item), 2),
            $this->renderIndent($this->renderListMenuItemDelete($item), 2),
        ]);

        return $render;
    }

    private function renderListMenuItemRead($item): string
    {
        $render = '';

        if (auth()->user()->can($this->can_prefix . '-read')) {

            if ($this->is_readable) {
                $template  = '<a class="dropdown-item" href="%s">' . PHP_EOL;
                $template .= '    <i class="fa fa-eye mr-1"></i>Voir' . PHP_EOL;
                $template .= '</a>' . PHP_EOL;

                if (empty($this->route_param)) {
                    $render = sprintf($template, route($this->route_prefix . '.' . 'read', array_merge([$item->id], Request::query())));
                } else {
                    $render = sprintf($template, route($this->route_prefix . '.' . 'read', array_merge([$this->route_param, $item->id ], Request::query())));
                }
            }
        }

        return $render;
    }

    private function renderListMenuItemUpdate($item): string
    {
        $render = '';

        if (auth()->user()->can($this->can_prefix . '-update')) {

            if ($this->is_editable) {
                $template  = '<a class="dropdown-item" href="%s">' . PHP_EOL;
                $template .= '    <i class="fa fa-pencil-square-o mr-1"></i>Modifier' . PHP_EOL;
                $template .= '</a>' . PHP_EOL;

                if (empty($this->route_param)) {
                    $render = sprintf($template, route($this->route_prefix . '.' . 'update', array_merge([$item->id], Request::query())));
                } else {
                    $render = sprintf($template, route($this->route_prefix . '.' . 'update', array_merge([$this->route_param, $item->id ], Request::query())));
                }
            }
        }

        return $render;
    }

    private function renderListMenuItemTag($item): string
    {
        $render = '';

        if (auth()->user()->hasRole('admin')) {

            if (count($this->_tags) > 0) {
                $template  = '<a class="dropdown-item clipboard-copy" href="#" title="Copier le Tag" data-clipboard-action="copy" data-clipboard-text="%s">' . PHP_EOL;
                $template .= '    <i class="fa fa-clipboard mr-1"></i>%s' . PHP_EOL;
                $template .= '</a>' . PHP_EOL;

                foreach ($this->_tags as $attr => $template_tag) {
                    $render_tag = sprintf($template_tag, data_get($item, $attr));
                    $render .= vsprintf($template, [$render_tag, $render_tag]);
                }
            }
        }

        return $render;
    }

    private function renderListMenuItemMore($item): string
    {
        $render = '';

        if (auth()->user()->can($this->can_prefix . '-read')) {

            $menu_items = $this->_menu_items;

            if (count($menu_items) > 0) {
                $template  = '<a class="dropdown-item" href="%s" target="%s">' . PHP_EOL;
                $template .= '    %s' . PHP_EOL;
                $template .= '</a>' . PHP_EOL;

                foreach ($menu_items as $menu_item) {

                    $href     = data_get($menu_item, 'href');
                    $name     = data_get($menu_item, 'name');
                    $callback = data_get($menu_item, 'callback');

                    $url = $this->objectToString($href, $name, $callback, $item);

                    $render .= vsprintf($template, [
                        $url,
                        $menu_item['target'] ?? '_self',
                        $menu_item['label']
                    ]);
                }
            }
        }

        return $render;
    }

    private function renderListMenuItemDuplicate($item): string
    {
        $render = '';

        if (auth()->user()->can($this->can_prefix . '-create')) {

            if ($this->is_duplicable) {
                $template = '<a class="dropdown-item" href="%s">' . PHP_EOL;
                $template .= '    <i class="fa fa-clone mr-1"></i>Dupliquer' . PHP_EOL;
                $template .= '</a>' . PHP_EOL;

                if (empty($this->route_param)) {
                    $render = sprintf($template, route($this->route_prefix . '.' . 'duplicate', array_merge([$item->id], Request::query())));
                } else {
                    $render = sprintf($template, route($this->route_prefix . '.' . 'duplicate', array_merge([$this->route_param, $item->id], Request::query())));
                }
            }

        }

        return $render;
    }

    private function renderListMenuItemDelete($item): string
    {
        $render = '';

        if (auth()->user()->can($this->can_prefix . '-delete')) {

            if ($this->is_deletable) {
                $template  = '<div class="dropdown-divider"></div>' . PHP_EOL;
                $template .= '<a class="dropdown-item delete" href="#" data-href="%s">' . PHP_EOL;
                $template .= '    <i class="fa fa-trash mr-1"></i>Supprimer' . PHP_EOL;
                $template .= '</a>' . PHP_EOL;

                if (empty($this->route_param)) {
                    $render = sprintf($template, route($this->route_prefix . '.' . 'delete', array_merge([$item->id], Request::query())));
                } else {
                    $render = sprintf($template, route($this->route_prefix . '.' . 'delete', array_merge([$this->route_param, $item->id], Request::query())));
                }
            }

        }

        return $render;
    }

    /**
     * Retourne le rende d'un paramètre class
     * @param  string $class Le contenu du paramètre
     * @return string        Le rendu du paramètre class
     */
    private function renderClass($class): string
    {
        $render = '';
        $template = ' class="%s"';

        if (!empty($class)) {
            $render = sprintf($template, $class);
        }

        return $render;
    }

    /**
     * Retourne le rendu d'attributs avec leur valeur
     * @param  array  $attrs Les attributs avec la colonne de l'object conteant la valeur
     * @param  Object $item  Object contenant la valeur de l'attribut
     * @return string        Le rendu des attributs avec leur valeur
     */
    private function renderAttributes($attrs = [], $item): string
    {
        $render = '';

        if (is_iterable($attrs)) {
            $template = ' %s="%s"';
            foreach ($attrs as $name => $value) {
                $render .= vsprintf($template, [$name, $this->objectToString(null, $value, null, $item)]);
            }
        }

        return $render;
    }

    /**
     * Retourne le rendu d'un template avec une liste de noms de colonnes
     * @param  string $template         Le template servant au rendu
     * @param  string|array $display    Le ou les nom de colonnes
     * @param  string $eval             Chaine de caractère à executer comme du php
     * @param  Mix $item                Objet source des donnés à afficher
     * @param  boolean $strip           Supprimer les balises HTML
     * @return string                   Le rendu
     */
    private function objectToString($template = null, $displays = null, $callback = null, $item, $strip = false)
    {
        // Création du texte à afficher
        $datas = [];

        // Convertion en tableau si besoin
        if (!is_iterable($displays)) {
           $displays = [$displays];
        }

        // Traitement du tableau de valeur à afficher
        foreach ($displays as $key => $display) {

            $data = mixte_get($item, $display);

            // Callback array
            if (is_array($callback) && isset($callback[$key]) && !is_null($callback[$key])) {
                $data = $callback[$key]($data);
            }

            $datas[] = $strip ? strip_tags($data) : $data;
        }

        // Traitement des % seuls pour éviter les conflits
        if (!empty($template)) {
            $template = preg_replace('/(%)[^0-9sd.\']/i', '%%"', $template);
        }

        // Traitement template & callback
        if (is_string($callback)) {
            if (empty($template)) {
                $string = call_user_func_array($callback, $datas);
            } else {
                $string = vsprintf($template, $datas);
                $string = $callback($string);
            }

        } else if (!empty($template)) {
            $string = vsprintf($template, $datas);
        } else {
            $string = $datas[0];
        }

        return $string;
    }

    /**
     * Retourne le rendu d'une chaine de caractère en HTML
     * @param  String $string La chains de caractères à traiter
     * @return String         Le rendu HTML
     */
    public function renderHTML($string): string
    {
        return new HtmlString($string);
    }

    /**
     * Retourne le rendu avec indentation d'un texte
     * @param  string $text  Le texte à traiter
     * @param  Integer $nb   Le nombre d'indentation à appliquer
     * @return string        Le rendu avec indentation
     */
    private function renderIndent($text, $nb): string
    {
        $indent = '    ';
        $render = '';

        $real_indent = '';

        for ($i = 1; $i <= $nb; $i++) {
            $real_indent .= $indent;
        }

        $lines = explode(PHP_EOL, $text);

        foreach ($lines as $line) {

            if (!empty($line)) {
                $render .= $real_indent . $line . PHP_EOL;
            }
        }

        return $render;
    }



    /**
     * Mise à jour des attributs depuis le tableau passé en paramètre
     * @param Array $attrs Liste des attributs à traiter
     * @return void
     */
    private function setAttrs($attrs)
    {
        $this->resetAttrs();
        if (is_null($attrs)) {
            return ;
        }

        // Liste des attributs autorisés
        $attrs_available = array_map(function ($val) {
            return trim($val, '_');
        }, array_keys(get_object_vars($this)));

        // Traitement des attributs
        if (is_array($attrs)) {
            foreach ($attrs as $attr => $value) {
                if (in_array($attr, $attrs_available)) {
                    if (is_array($value)) {
                        $this->{'_' . $attr} = $value;
                    } else {
                        $this->{$attr} = $value;
                    }
                } else {
                    throw new \InvalidArgumentException("Le paramètre [" . $attr . "] n'est pas pris en charge");
                }
            }
        } else {
            throw new \InvalidArgumentException("Type de paramètre invalide. Attendu : Tableau");
        }

    }

    /**
     * Réinitialisation des attributs de constructions
     * @return void
     */
    private function resetAttrs()
    {
        $attrs = array_keys(get_object_vars($this));

        foreach ($attrs as $attr) {

            if (preg_match('/^_.*/', $attr)) {
                $this->{$attr} = [];
            } else if (preg_match('/^is_.*/', $attr)) {
                $this->{$attr} = false;
            } else {
                $this->{$attr} = null;
            }
        }
    }
}
